
local emptyFunc = function(arg1) end
local _G = _G
local AMF = AMF
AMF.Filter = {}

-- table recycler
local tableRepo = {}
function AMF:NewTable()
	if next(tableRepo) then
		return table.remove(tableRepo)
	else
		return {}
	end
end

function AMF:RecycleTable(t)
	for k in pairs(t) do
		if type(t[k]) == 'table' then
			t[k] = self:RecycleTable(t[k])
		else
			t[k] = nil
		end
	end
	tableRepo[#tableRepo+1] = t
	return nil
end

function AMF:WipeTable(t)
	for k in pairs(t) do
		if type(t[k]) == 'table' then
			t[k] = self:WipeTable(t[k])
			tableRepo[#tableRepo+1] = t[k]
			t[k] = nil
		else
			t[k] = nil
		end
	end
	return t
end

AMF.FilterDropDownMenu = CreateFrame("Frame", "AMF_FilterDropDownMenu")
AMF.FilterDropDownMenu.displayMode = "MENU"
AMF.FilterDropDownMenu.HideMenu = function()
    if UIDROPDOWNMENU_OPEN_MENU == AMF.FilterDropDownMenu then
        CloseDropDownMenus()
    end
end
local info = {}
AMF.FilterDropDownMenu.initialize = function(self, level)
	if not level then return end
	wipe(info)
	info.keepShownOnClick = 1
	if level == 1 then
		local filters = 1
		AMF:SetupFilters()
		-- Create the title of the menu
		info.isTitle = 1
		info.text = "Filter "..filters..':'
		info.notCheckable = 1
		UIDropDownMenu_AddButton(info, level)
		
		-- reset info table
        info.disabled = nil
        info.isTitle = nil
        info.notCheckable = nil
		for i=1, #AMF.FilterTypes do
			info.text = AMF.FilterTypes[i]
			if AMF.FilterSubTypes[AMF.FilterTypes[i]] and (#AMF.FilterSubTypes[AMF.FilterTypes[i]] > 1) then
				info.hasArrow = true
			else
				info.hasArrow = nil
			end
			info.arg1 = AMF.FilterTypes[i]
			info.value = AMF.FilterTypes[i]
			info.func = AMF.FilterToggle
			if AMF.FilterTypes[i] == USABLE_ITEMS then
				info.checked = AMF.Filter_USABLE_ITEMS
			elseif AMF.FilterTypes[i] == TUTORIAL_TITLE24 then
				info.checked = AMF.Filter_EQUIPPABLE_ITEMS
			else
				info.checked = AMF.Filter[AMF.FilterTypes[i]]
			end
			UIDropDownMenu_AddButton(info, level)
			if AMF.FilterTypes[i] == USABLE_ITEMS or AMF.FilterTypes[i] == TUTORIAL_TITLE24 then
				filters = filters + 1
				wipe(info)
				info.disabled = nil
				info.isTitle = 1
				info.text = "Filter "..filters..':'
				info.notCheckable = 1
				UIDropDownMenu_AddButton(info, level)
				info.isTitle = nil
				info.notCheckable = nil
				info.disabled = nil
				info.keepShownOnClick = 1
			end
		end
		-- Close menu item
		info.text = CLOSE
		info.func = function() CloseDropDownMenus() end
		info.checked = nil
		info.hasArrow = nil
		info.notCheckable = 1
		UIDropDownMenu_AddButton(info, level)
	elseif level == 2 then
		for i=1, #AMF.FilterSubTypes[UIDROPDOWNMENU_MENU_VALUE] do
			info.text = AMF.FilterSubTypes[UIDROPDOWNMENU_MENU_VALUE][i]
			info.arg1 = UIDROPDOWNMENU_MENU_VALUE
			info.arg2 = AMF.FilterSubTypes[UIDROPDOWNMENU_MENU_VALUE][i]
			info.func = AMF.FilterToggle
			if UIDROPDOWNMENU_MENU_VALUE == TUTORIAL_TITLE24 then
				if AMF.Filter_EQUIPPABLE_ITEMS and AMF.Filter_EQUIPPABLE_ITEMS[AMF.FilterSubTypes[UIDROPDOWNMENU_MENU_VALUE][i]] then
					info.checked = true
				else
					info.checked = nil
				end
			else
				if AMF.Filter[UIDROPDOWNMENU_MENU_VALUE] and AMF.Filter[UIDROPDOWNMENU_MENU_VALUE][AMF.FilterSubTypes[UIDROPDOWNMENU_MENU_VALUE][i]] then
					info.checked = true
				else
					info.checked = nil
				end
			end
			UIDropDownMenu_AddButton(info, level)
		end
	end
end

function AMF.FilterToggle(dropdownbutton, arg1, arg2, checked)
	local self = AMF
	if arg1 == USABLE_ITEMS then
		if checked then
			AMF.Filter_USABLE_ITEMS = true
		else
			AMF.Filter_USABLE_ITEMS = false
		end
	elseif arg1 == TUTORIAL_TITLE24 then
		if checked then
			self.Filter_EQUIPPABLE_ITEMS = self.Filter_EQUIPPABLE_ITEMS or AMF:NewTable()
			if arg2 then
				self.Filter_EQUIPPABLE_ITEMS[arg2] = true
			end
		else
			if arg2 then
				self.Filter_EQUIPPABLE_ITEMS[arg2] = nil
			else
				self.Filter_EQUIPPABLE_ITEMS = AMF:RecycleTable(self.Filter_EQUIPPABLE_ITEMS)
				CloseDropDownMenus(2)
			end
		end
	else
		if checked then
			self.Filter[arg1] = self.Filter[arg1] or AMF:NewTable()
			if arg2 then
				self.Filter[arg1][arg2] = true
			end
		else
			if arg2 then
				self.Filter[arg1][arg2] = nil
			else
				self.Filter[arg1] = AMF:RecycleTable(self.Filter[arg1])
				CloseDropDownMenus(2)
			end
		end
	end
	AMF:SetupMerchantFrame()
end

--local TUTORIAL_TITLE24 = "Equippable Items" -- EQUIPPABLE_ITEMS

AMF.FilterTypes = {}
AMF.FilterSubTypes = {}
AMF.FilterSubTypesFound = {}
AMF.EquipLocFound = {}
function AMF:SetupFilters()
	local numMerchantItems = GetMerchantNumItems()
	self.FilterTypes = AMF:WipeTable(self.FilterTypes)
	self.FilterSubTypes = AMF:WipeTable(self.FilterSubTypes)
	self.FilterSubTypesFound = AMF:WipeTable(self.FilterSubTypesFound)
	self.EquipLocFound = AMF:WipeTable(self.EquipLocFound)
	local usable
	local notusable
	for i=1,numMerchantItems do
		if GetMerchantItemLink(i) then
			local _, _, _, _, _, isUsable = GetMerchantItemInfo(i)
			local _, _, _, _, _, itemType, itemSubType, _, itemEquipLoc = GetItemInfo(GetMerchantItemLink(i))
			if isUsable then
				usable = true
			else
				notusable = true
			end
			if not self.FilterSubTypes[itemType] then
				self.FilterTypes[#self.FilterTypes+1] = itemType
				self.FilterSubTypes[itemType] = AMF:NewTable()
			end
			if not self.FilterSubTypesFound[itemSubType] then
				self.FilterSubTypes[itemType][#self.FilterSubTypes[itemType]+1] = itemSubType
				self.FilterSubTypesFound[itemSubType] = true
			end
			if itemEquipLoc ~= '' and not self.EquipLocFound[itemEquipLoc] then
				if not self.FilterSubTypes[TUTORIAL_TITLE24] then
					self.FilterTypes[#self.FilterTypes+1] = TUTORIAL_TITLE24
					self.FilterSubTypes[TUTORIAL_TITLE24] = AMF:NewTable()
				end
				if not self.EquipLocFound[itemEquipLoc] then
					self.FilterSubTypes[TUTORIAL_TITLE24][#self.FilterSubTypes[TUTORIAL_TITLE24]+1] = _G[itemEquipLoc]
					self.EquipLocFound[itemEquipLoc] = true
				end
				
			end
		end
	end
	if usable and notusable then
		self.FilterTypes[#self.FilterTypes+1] = USABLE_ITEMS
		self.FilterSubTypes[USABLE_ITEMS] = AMF:NewTable()
	end
	if self.FilterSubTypes[ARMOR] and #self.FilterSubTypes[ARMOR] == 1 then
		self.FilterSubTypes[ARMOR] = nil
		for i=1, #self.FilterTypes do
			if self.FilterTypes[i] == ARMOR then
				table.remove(self.FilterTypes, i)
				break
			end
		end
	end
	table.sort(AMF.FilterTypes, function (a, b)
		if a == USABLE_ITEMS and b == TUTORIAL_TITLE24 then
			return true
		elseif b == USABLE_ITEMS and a == TUTORIAL_TITLE24 then
			return false
		elseif a == USABLE_ITEMS or a == TUTORIAL_TITLE24 then
			return true
		elseif b == USABLE_ITEMS or b == TUTORIAL_TITLE24 then
			return false
		else
			return a < b
		end
    end)
	for k in pairs(self.FilterSubTypes) do
		table.sort(self.FilterSubTypes[k])
	end
end
